package golfCourse.agents.databaseAgent;

import golfCourse.agents.GolfCourseAgent;
import golfCourse.agents.SendSystemErrorBehaviour;
import golfCourse.communication.ontology.GolfCourseOntology;
import golfCourse.ontologies.jastor.golfCourse.GolfCourse;
import golfCourse.ontology.storage.ModelLoader;
import golfCourse.ontology.storage.ModelLoaderInitException;
import golfCourse.properties.GolfCourseSystemProperties;
import jade.content.lang.sl.SLCodec;
import jade.core.AID;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPANames;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.MessageTemplate;
import jade.util.Logger;

import java.util.logging.Level;

import com.hp.hpl.jena.ontology.OntModel;

/**
 * Database agent for insert, update and remove data from database.
 * Only one DatabaseAgent is in Golf System.
 */
public class DataBaseAgent extends GolfCourseAgent {
	
	private static Logger logger = Logger.getMyLogger(DataBaseAgent.class.getName());
	
	public final static String SERVICE_NAME = "Save Golf Course Reservations Services";
	public final static String SERVICE_TYPE = "save-service";
	public final static String SERVICE_OWNERSHIP = "Golf System";

	// ontology model describing domain of golf course world 
	private OntModel mDomain;
	
	// ontology model with instances of golf course (from database).
    private OntModel mGolfCoursesDB;
	
    public OntModel getDomainModel()
    {
    	return mDomain;
    }
    
    public OntModel getGolfCoursesDBModel()
    {
    	return mGolfCoursesDB;
    }
    
    public DataBaseAgent()
    {
    	super();
    }
    
    /**
	 * Loads ontology models.
	 */
	private void loadOntologyModels() {
		logger.info("Database agent: loading golf course ontology ... ");
			
		mDomain = ModelLoader.loadModel(GolfCourseSystemProperties.DOMAIN_GOLF_COURSE);
		logger.info("Database Agent: golf course domain model ise loaded");
			
		logger.info("Database Agent: loading golf courses db from DB ...");
		try{
			mGolfCoursesDB = ModelLoader.loadDBModel(GolfCourse.TYPE.getNameSpace(), GolfCourseSystemProperties.DB_GOLF_COURSE);
			logger.info("Database Agent: DBModel is loaded");
		} catch (ModelLoaderInitException e) {
			this.addBehaviour(new SendSystemErrorBehaviour("Database Agent: problem while initializing ModelLoader: " + e.getMessage(), this));
			logger.info("Database Agent: problem while initializing ModelLoader: " + e.getLocalizedMessage());
			//doDelete();			
		}
	}
	
	protected void setup() {
		getManagerAID();
		
		getContentManager().registerLanguage(new SLCodec(), FIPANames.ContentLanguage.FIPA_SL0);
		getContentManager().registerOntology(GolfCourseOntology.getInstance());

		MessageTemplate mt = MessageTemplate
				.MatchOntology(GolfCourseOntology.ONTOLOGY_NAME);
		
		addBehaviour(new DataBaseAgentResponder(this, mt));

		loadOntologyModels();
		
		registerInDF();
	
		logger.info(getLocalName() + " is created");
	}
	
	/**
	 * Register in DF.
	 *
	 */
	protected void registerInDF() {
		try {
			ServiceDescription sd = getServiceDescription();			
			DFAgentDescription dfd = new DFAgentDescription();
			dfd.setName(getAID());
			dfd.addServices(sd);
			DFService.register(this, dfd);
			logger.info(getLocalName() + " is register");
		} catch (FIPAException e) {
			logger.info(getLocalName() + " cannot register database agent at DF: " + e.getLocalizedMessage());
			doDelete();
		}
	}
	
	/**
	 * Returns information about the service this agent provides
	 * 
	 * @return service description
	 */
	public static ServiceDescription getServiceDescription() {
		ServiceDescription sd = new ServiceDescription();
		sd.setType(SERVICE_TYPE);
		sd.setName(SERVICE_NAME);
		sd.setOwnership(SERVICE_OWNERSHIP);
		return sd;
	}

	/**
	 * Database agent is created by ManagerAgent.
	 * ManagerAgent set his AID in parameter.
	 */
	protected void getManagerAID()
	{
		Object[] args = getArguments();
		if(args!= null)
		{
			if(args.length > 0)
			{
				this.managerAID = (AID)args[0];
				return;
			}
		}
	}
	
	protected void takeDown() 
    {
		try {
			unregisterInDF();
			logger.info(getLocalName() + " is unregistered");
		}
		catch(FIPAException e)
		{
			logger.info(getLocalName() + " cannot be unregistered: " + e.getLocalizedMessage());
		}
    }
}
