package golfCourse.agents.translationAgent;

import golfCourse.agents.GolfCourseAgent;
import golfCourse.agents.contactToOTASystemAgent.ContactToOTASystemAgent;
import golfCourse.agents.databaseAgent.DataBaseAgent;
import golfCourse.agents.searchAgent.SearchAgent;
import golfCourse.communication.ontology.GolfCourseOntology;
import golfCourse.communication.services.ServiceFinder;
import golfCourse.communication.services.ServiceFinderException;
import golfCourse.ontology.storage.DomainModelsFactory;
import jade.content.lang.sl.SLCodec;
import jade.core.AID;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPANames;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.MessageTemplate;
import jade.util.Logger;

import com.hp.hpl.jena.ontology.OntModel;

public class TranslationAgent extends GolfCourseAgent{

	private static Logger logger = Logger.getMyLogger(TranslationAgent.class.getName());
	
	public final static String SERVICE_NAME = "Translation Services";
	public final static String SERVICE_TYPE = "translation-service";
	public final static String SERVICE_OWNERSHIP = "Golf Course System";
	
	public ServiceFinder searchingService;
	public ServiceFinder savingService;
	public ServiceFinder cotactWithOTASystemsService;
		
	public OntModel domainGolfCourseModel;
	public OntModel domainContactsModel;
	public OntModel domainAddressModel;
	
	public OntModel domainGolfCourseTeeTimeModel;
	public OntModel domainFeeModel;
	public OntModel domainPriceModel;
	public OntModel domainDescriptionModel;
	
	public TranslationAgent()
	{
		super();
	}
	
	public void setup()
	{
		getManagerAID();
				
		getContentManager().registerLanguage(new SLCodec(), FIPANames.ContentLanguage.FIPA_SL0);
		getContentManager().registerOntology(GolfCourseOntology.getInstance());

		MessageTemplate mt = MessageTemplate
				.MatchOntology(GolfCourseOntology.ONTOLOGY_NAME);
		
		addBehaviour(new TranslationAgentResponder(this, mt));

		loadDomainModels();
		
		registerInDF();
		
		logger.info(getLocalName() + " is created");
		
		findServices();	
	}
	
	private void findServices()
	{
		try{
			searchingService = new ServiceFinder(this, SearchAgent.getServiceDescription());
		}catch(ServiceFinderException e)
		{
			logger.info(getLocalName() + ": no search agents were found");
			doDelete();
			return;
		}
		
		try {
			savingService = new ServiceFinder(this, DataBaseAgent.getServiceDescription());
		}catch(ServiceFinderException e)
		{
			logger.info(getLocalName() + ": no database agent was found.");
			doDelete();
			return;
		}
		
		try{
			cotactWithOTASystemsService = new ServiceFinder(this, ContactToOTASystemAgent.getServiceDescription());
		}
		catch(ServiceFinderException e)
		{
			logger.info(getLocalName() + ": no contact to OTASystem agent was found.");
			doDelete();
			return;
		}
	}
	
	private void loadDomainModels()
	{
		this.domainGolfCourseModel = DomainModelsFactory.getGolfCourseDomain();
		this.domainAddressModel = DomainModelsFactory.getAddressDomain();
		this.domainContactsModel = DomainModelsFactory.getContactsDomain();
		this.domainDescriptionModel = DomainModelsFactory.getDescriptionModel();
		this.domainFeeModel = DomainModelsFactory.getFeeModel();
		this.domainPriceModel = DomainModelsFactory.getPriceModel();
		this.domainGolfCourseTeeTimeModel = DomainModelsFactory.getGolfCourseTeeTimeModel();
	}
	
	protected void registerInDF()
	{
		try {
			ServiceDescription sd = getServiceDescription();			
			DFAgentDescription dfd = new DFAgentDescription();
			dfd.setName(getAID());
			dfd.addServices(sd);
			DFService.register(this, dfd);
			logger.info(getLocalName() + " is register");
		} catch (FIPAException e) {
			logger.info(getLocalName() + ": cannot register "+ getLocalName() + " at DF - " + e.getMessage());
			doDelete();
		}
	}
	
	protected void getManagerAID()
	{
		Object[] args = getArguments();
		if(args!= null)
		{
			if(args.length > 0)
			{
				this.managerAID = (AID)args[0];
				return;
			}
		}
	}
	
	/**
	 * Returns information about the service this agent provides
	 * 
	 * @return service description
	 */
	public static ServiceDescription getServiceDescription() {
		ServiceDescription sd = new ServiceDescription();
		sd.setType(SERVICE_TYPE);
		sd.setName(SERVICE_NAME);
		sd.setOwnership(SERVICE_OWNERSHIP);
		return sd;
	}
	
	protected void takeDown() 
    {
		try {
			unregisterInDF();
			logger.info(getLocalName() + " is unregistered");
		}
		catch(FIPAException e)
		{
			logger.info(getLocalName() + " cannot be unregistered: " + e.getMessage());
		}
    }
}
